###################################################
# R code for "Big Data: New Tricks for Econometrics
# Journal of Economic Perspectives 28(2), 3-28            
# http://pubs.aeaweb.org/doi/pdfplus/10.1257/jep.28.2.3
# Hal R. Varian
###################################################
library(glmnet)
dat <- read.csv("/home/hal/Working/JEP/Lasso/FLS-data.csv")

# for reproducability
set.seed(1234)

# arrange data and estimate
x <- as.matrix(dat[,-1])
y <- dat[,1]
model.lasso <- glmnet(x,y)

# cross validation to pick lambda
cv.out <- cv.glmnet(x,y)
plot(cv.out)
# pick best lambda
best.lambda <- cv.out$lambda.min
best.lambda
# convert to logs to compare to plot
log(best.lambda)

# estimate model and look at coefficients
model.coef <- predict(model.lasso,s=best.lambda,type="coef")[1:42,]
model.coef.ne0 <- model.coef[model.coef != 0]
abs.order <- order(abs(model.coef.ne0))
rev(model.coef.ne0[abs.order])

# compute prediction
y.pred <- predict(model.lasso,s=best.lambda,newx=x)
error <- abs((y-y.pred)/y)
mean(error)

# note huge outlier at obs 38
# what country is it?
mean(error[-38])
